<?php
/**
 * Project:     PHPLinkDirectory: Link exchange directory
 *
 * License: GNU GPL (http://www.opensource.org/licenses/gpl-license.html)
 * 
 * This program is free software;
 * 
 * you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * For questions, help, comments, discussion, etc., please join the
 * PHP Link Directory Forum http://www.phplinkdirectory.com/forum/

 *
 * @link http://www.phplinkdirectory.com/
 * @copyright 2004-2005 NetCreated, Inc. (http://www.netcreated.com/)
 * @projectManager David DuVal <david@david-duval.com>
 * @author Bogdan Dumitru <dcb@insomniacsoft.com>
 * @mod developers and support
 * 		Casey Wilson / Ap0s7le <at@ap0s7le.com>
 * 		York Kie Tan / yktan <yktan@hotmail.com>
 * @package PHPLinkDirectory
 * @version 2.0.0 RC5.2
 **/

require_once 'include/tables.php';
require_once 'libs/adodb/adodb.inc.php';

/**
 * creates or updates the database structure based on the structure defined in tables.php 
 *
 * @param string $db_type database type
 * @param string $db_host database host
 * @param string $db_name dabase name
 * @param string $db_user database login
 * @param string $db_password database password
 * @return int 0 if succesfull, error code otherwise
 */

function create_db($db_type, $db_host, $db_name, $db_user, $db_password) {
	global $tables;
	$db = ADONewConnection($db_type);
	$db_created = 0;
	if (!$db->Connect($db_host, $db_user, $db_password)) {
		return array (false, 'INSTALL_ERROR_CONNECT', $db->ErrorMsg());
	}
	$db = ADONewConnection($db_type);
	if (!$db->Connect($db_host, $db_user, $db_password, $db_name)) {
		$db = ADONewConnection($db_type);
		if ($db->Connect($db_host, $db_user, $db_password)) {
			$dict = NewDataDictionary($db);
			$sql_array = $dict->CreateDatabase($db_name);
			if ($sql_array) {
				$db_created = $dict->ExecuteSQLArray($sql_array);
			}
		}
		if ($db_created != 2) {
			return array (false, 'INSTALL_ERROR_CREATE_DB', $db->ErrorMsg());
		}
		$db->SelectDB($db_name);
	}
	#$db->debug = true;
	$tables_existing = $db->MetaTables('TABLES');
	$dict = NewDataDictionary($db);
	foreach ($tables as $table_key => $table) {
		$table_name = $table['name'];
		if (is_array($table['fields'])) {
			/*if (in_array($table_name, $tables_existing)||in_array(strtolower($table_name), $tables_existing)) {
				$columns_existing = $db->MetaColumnNames($table_name);
				$fields = array();
				foreach ($table['fields'] as $field_name => $field_def) {
					if (array_key_exists($field_name, $columns_existing))
						continue;
					$fields[] = $field_name.' '.$field_def;
				}
				if(count($fields)>0){
					$sql_array = $dict->AddColumnSQL($table_name, implode(',', $fields));
					if (!$dict->ExecuteSQLArray($sql_array)) {
						return array (false, 'INSTALL_ERROR_CREATE', $db->ErrorMsg());
					}
				}
			} else {*/
				$fields = array ();
				foreach ($table['fields'] as $field_name => $field_def) {
					$fields[] = $field_name.' '.$field_def;
				}
				$created = 0;
				if ($sql_array = $dict->ChangeTableSQL($table_name, implode(',', $fields))) {
					$created = $dict->ExecuteSQLArray($sql_array);
				}
				if ($created != 2)
					return array (false, 'INSTALL_ERROR_CREATE', $db->ErrorMsg());
			//}
		}
		if (is_array($table['indexes'])) {
			$indexes_existing = $db->MetaIndexes($table_name);
			foreach ($table['indexes'] as $index_name => $index_def) {
				$index_name = $table_name.'_'.$index_name.'_IDX';
				$index_opts = array ();
				if (is_array($index_def)) {
					$index_fields = $index_def[0];
					$index_opts = explode(' ', $index_def[1]);
				} else {
					$index_fields = $index_def;
				}
				if(array_key_exists($index_name, $indexes_existing)||array_key_exists(strtolower($index_name), $indexes_existing)){
					if ($sql_array = $dict->CreateIndexSQL($index_name, $table_name, $index_fields, array_merge($index_opts, array ('DROP')))) {
						$dict->ExecuteSQLArray($sql_array);
					}
				}
				$created = 0;
				if ($sql_array = $dict->CreateIndexSQL($index_name, $table_name, $index_fields, $index_opts)) {
					$created = $dict->ExecuteSQLArray($sql_array);
				}
				if ($created != 2)
					return array (false, 'INSTALL_ERROR_CREATE', $db->ErrorMsg());
			}
		}
		if (is_array($table['data'])) {
			foreach ($table['data'] as $row) {
				$sql = "SELECT ID FROM $table_name WHERE ID = '{$row['ID']}'";
				$rs = $db->SelectLimit($sql, 1);
				if ($rs && $rs->EOF) {
					if (!$db->AutoExecute($table_name, $row, 'INSERT', false, true, true))
						return array (false, 'INSTALL_ERROR_CREATE', $db->ErrorMsg());
				}
			}
		}
	}
	return array (true, $db_created == 0 ? 'INSTALL_DB_UPDATED' : 'INSTALL_DB_CREATED');
}

function update_config($file_name, $values) {
	if (!INSTALL_PATH.file_exists($file_name))
		return 'CONFIG_NOT_FOUND';
	if (!is_writable(INSTALL_PATH.$file_name))
		return 'CONFIG_NOT_WRITABLE';
	$file = file_get_contents(INSTALL_PATH.$file_name);
	$vals = '';
	foreach ($values as $key => $val) {
		if (!preg_match("`define\s*\(\s*(?:'|\")$key(?:'|\")\s*,\s*(?:'|\")?.*(?:'|\")?\s*\);`Um", $file)) {
			$vals .= "define('$key', '$val');\n";

		} else {
			$file = preg_replace("`define\s*\(\s*(?:'|\")$key(?:'|\")\s*,\s*(?:'|\")?.*(?:'|\")?\s*\);`Um", "define('$key', '$val');", $file);
		}
	}
	$insert_point = strrpos($file, '?>');
	if ($insert_point !== false) {
		$file = substr($file, 0, $insert_point).$vals.substr($file, $insert_point);
	}
	$f = fopen(INSTALL_PATH.$file_name, 'w');
	fwrite($f, $file);
	fclose($f);
	return true;
}
?>