<?php

/**
  * Based on:
 
  * PHP-Class hn_captcha Version 1.2, released 16-Apr-2004
  * Author: Horst Nogajski, horst@nogajski.de
  * 
  * License: GNU GPL (http://www.opensource.org/licenses/gpl-license.html)
  * Download: http://hn273.users.phpclasses.org/browse/package/1569.html
  *
  * If you find it useful, you might rate it on http://www.phpclasses.org/rate.html?package=1569
  * If you use this class in a productional environment, you might drop me a note, so I can add a link to the page.
  **/

require_once 'filters.class.php';

class captcha
{
		/**
          * @shortdesc Absolute path to folder with TrueTypeFonts (with trailing slash!). This must be readable by PHP.
		  * @type string
		  * @public
          *
          **/
		var $TTF_folder;
		
		/**
          * @shortdesc A List with available TrueTypeFonts for random char-creation.
		  * @type mixed[array|string]
		  * @public
          *
          **/
		var $TTF_RANGE  = array();

		/**
          * @shortdesc How many chars the generated text should have
		  * @type integer
		  * @public
          *
          **/
		var $chars		= 6;

		/**
          * @shortdesc The minimum size a Char should have
		  * @type integer
		  * @public
          *
          **/
		var $minsize	= 20;

		/**
          * @shortdesc The maximum size a Char can have
		  * @type integer
		  * @public
          *
          **/
		var $maxsize	= 40;

		/**
          * @shortdesc The maximum degrees a Char should be rotated. Set it to 30 means a random rotation between -30 and 30.
		  * @type integer
		  * @public
          *
          **/
		var $maxrotation = 30;

		/**
          * @shortdesc Background noise On/Off (if is Off, a grid will be created)
		  * @type boolean
		  * @public
          *
          **/
		var $noise		= TRUE;

		/**
          * @shortdesc This will only use the 216 websafe color pallette for the image.
		  * @type boolean
		  * @public
          *
          **/
		var $websafecolors = FALSE;

	////////////////////////////////
	//
	//	PRIVATE PARAMS
	//

		/** @private **/
		var $lx;				// width of picture
		/** @private **/
		var $ly;				// height of picture
		/** @private **/
		var $jpegquality = 80;	// image quality
		/** @private **/
		var $noisefactor = 9;	// this will multiplyed with number of chars
		/** @private **/
		var $nb_noise;			// number of background-noise-characters
		/** @private **/
		var $TTF_file;			// holds the current selected TrueTypeFont
		/** @private **/
		var $gd_version;		// holds the Version Number of GD-Library
		/** @private **/
		var $r;
		/** @private **/
		var $g;
		/** @private **/
		var $b;

	////////////////////////////////
	//
	//	CONSTRUCTOR
	//

		/**
		  * @shortdesc Extracts the config array and generate needed params.
		  * @private
		  * @type void
		  * @return nothing
		  *
		  **/
		function captcha($config,$secure=TRUE)
		{

			// Test for GD-Library(-Version)
			$this->gd_version = $this->get_gd_version();
			if($this->gd_version == 0) die("There is no GD-Library-Support enabled. The Captcha-Class cannot be used!");

			// extracts config array
			if(is_array($config))
			{
				if($secure && strcmp('4.2.0', phpversion()) < 0)
				{
					$valid = get_class_vars(get_class($this));
					foreach($config as $k=>$v)
					{
						if(array_key_exists($k,$valid)) $this->$k = $v;
					}
				}
				else
				{
					foreach($config as $k=>$v) $this->$k = $v;
				}
			}

			// check TrueTypeFonts
			if(is_array($this->TTF_RANGE))
			{
				$temp = array();
				foreach($this->TTF_RANGE as $k=>$v)
				{
					if(is_readable($this->TTF_folder.$v)) $temp[] = $v;
				}
				$this->TTF_RANGE = $temp;
				if(count($this->TTF_RANGE) < 1) die('No Truetypefont available for the CaptchaClass.');
			}
			else
			{
				if(!is_readable($this->TTF_folder.$this->TTF_RANGE)) die('No Truetypefont available for the CaptchaClass.');
			}

			// select first TrueTypeFont
			$this->change_TTF();


			// get number of noise-chars for background if is enabled
			$this->nb_noise = $this->noise ? ($this->chars * $this->noisefactor) : 0;

			// set dimension of image
			if(empty($this->lx)){
				$this->lx = ($this->chars + 1) * (int)(($this->maxsize + $this->minsize) / 1.5);
			}
			if(empty($this->ly)){
				$this->ly = (int)(2.4 * $this->maxsize);
			}

			$this->value = '';
			for($i=0;$i<$this->chars;$i++){
				$this->value .= chr(mt_rand(65, 90));
			}

		}
		
		function imagecolorgradient($img,$x1,$y1,$height,$width,$colA,$colB) {
			$varC1=($colA[1]-$colB[1])/$height;
			$varC2=($colA[2]-$colB[2])/$height;
			$varC3=($colA[3]-$colB[3])/$height;
			for ($i=0;$i<=$height;$i++) {
				$col=ImageColorAllocate($img,
				$colA[1]-floor($i*$varC1),
				$colA[2]-floor($i*$varC2),
				$colA[3]-floor($i*$varC3));
				ImageLine($img,$x1,$y1+$i,$x1+$width,$y1+$i,$col);
				}
			}

		function make_captcha()
		{
			// create Image and set the apropriate function depending on GD-Version & websafecolor-value
			if($this->gd_version >= 2 && !$this->websafecolors)
			{
				$func1 = 'imagecreatetruecolor';
				$func2 = 'imagecolorallocate';
			}
			else
			{
				$func1 = 'imageCreate';
				$func2 = 'imagecolorclosest';
			}
			$image = $func1($this->lx,$this->ly);

			// Set Background
			$y2 = 0;
			$this->random_color(120, 200);
			$colB = array(1 => $this->r, 2 => $this->g, 3 => $this->b);
			do{
			$y1 = $y2+1;
			$y2 += mt_rand(10, 20);
			$colA = $colB;
			$this->random_color(120, 200);
			$colB = array(1 => $this->r, 2 => $this->g, 3 => $this->b);
			$this->imagecolorgradient($image, 0, $y1, $y2-$y1, $this->lx, $colA, $colB);
			}while($y2<$this->ly);


			// allocates the 216 websafe color palette to the image
			if($this->gd_version < 2 || $this->websafecolors) $this->makeWebsafeColors($image);


			// fill with noise or grid
			if($this->nb_noise > 0)
			{
				// random characters in background with random position, angle, color
				for($i=0; $i < $this->nb_noise; $i++)
				{
					srand((double)microtime()*1000000);
					$size	= intval(rand((int)($this->minsize / 2.3), (int)($this->maxsize / 1.7)));
					srand((double)microtime()*1000000);
					$angle	= intval(rand(0, 20));
					srand((double)microtime()*1000000);
					$x		= intval(rand(0, $this->lx));
					srand((double)microtime()*1000000);
					$y		= intval(rand(0, (int)($this->ly - ($size / 5))));
					$this->random_color(120	, 190);
					$color	= $func2($image, $this->r, $this->g, $this->b);
					srand((double)microtime()*1000000);
					$text	= chr(intval(rand(48,90)));
					@ImageTTFText($image, $size, $angle, $x, $y, $color, $this->change_TTF(), $text);
				}
			}
			// generate Text
			for($i=0, $x = intval(rand($this->minsize,$this->maxsize))/2; $i < $this->chars; $i++)
			{
				$text	= strtoupper(substr($this->value, $i, 1));
				mt_srand((double)microtime()*1000000);
				$angle	= intval(mt_rand(($this->maxrotation * -1), $this->maxrotation));
				mt_srand((double)microtime()*1000000);
				$size	= intval(mt_rand($this->minsize, $this->maxsize));
				mt_srand((double)microtime()*1000000);
				$tl = imagettfbbox ($size, $angle, $this->change_TTF(), $text);
				
				$y = ($tl[3]-$tl[7]) + 20; //intval(rand((int)($size * 1.9), (int)($this->ly - ($size / 2))));
				$this->random_color(0, 127);
				$color	=  $func2($image, $this->r, $this->g, $this->b);
				$this->random_color(190, 255);
				$shadow = $func2($image, $this->r, $this->g, $this->b);
				@ImageTTFText($image, $size, $angle, $x + (int)($size / 20), $y, $shadow, $this->TTF_file, $text);
				@ImageTTFText($image, $size, $angle, $x, $y - (int)($size / 20), $color, $this->TTF_file, $text);
				
				$x += ($tl[4] - $tl[0]) + (int)(mt_rand(5, 10));
			}
			
			// generate lines
				for($i=0 ; $i < $this->ly; $i += (int)($this->minsize / 2))
				{
					$this->random_color(0, 127);
					$color	= $func2($image, $this->r, $this->g, $this->b);
					@imageline($image, mt_rand(0, 20), mt_rand(0, $this->ly), $this->lx-mt_rand(0, 20), $i, $color);
				}
				
			$im = new Wavy;
            $im->setImage($image);
            $im->run();
            $image = $im->getImage();
			header('Content-Type: image/jpeg');
			header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
			header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
			header("Cache-Control: no-store, no-cache, must-revalidate");
			header("Cache-Control: post-check=0, pre-check=0", false);
			header("Pragma: no-cache");
			@ImageJPEG($image, '', $this->jpegquality);
			@ImageDestroy($image);
		}

		/** @private **/
		function makeWebsafeColors(&$image)
		{
			//$a = array();
			for($r = 0; $r <= 255; $r += 51)
			{
				for($g = 0; $g <= 255; $g += 51)
				{
					for($b = 0; $b <= 255; $b += 51)
					{
						$color = imagecolorallocate($image, $r, $g, $b);
					}
				}
			}
		}
		
		/** @private **/
		function random_color($min,$max)
		{
			srand((double)microtime() * 1000000);
			$this->r = intval(rand($min,$max));
			srand((double)microtime() * 1000000);
			$this->g = intval(rand($min,$max));
			srand((double)microtime() * 1000000);
			$this->b = intval(rand($min,$max));
		}

		/** @private **/
		function change_TTF()
		{
			if(is_array($this->TTF_RANGE))
			{
				srand((float)microtime() * 10000000);
				$key = array_rand($this->TTF_RANGE);
				$this->TTF_file = $this->TTF_folder.$this->TTF_RANGE[$key];
			}
			else
			{
				$this->TTF_file = $this->TTF_folder.$this->TTF_RANGE;
			}
			return $this->TTF_file;
		}

		/** @private **/
		function get_gd_version()
		{
			static $gd_version_number = null;
			if($gd_version_number === null)
			{
			   ob_start();
			   phpinfo(8);
			   $module_info = ob_get_contents();
			   ob_end_clean();
			   if(preg_match("/\bgd\s+version\b[^\d\n\r]+?([\d\.]+)/i", $module_info, $matches))
			   {
				   $gd_version_number = $matches[1];
			   }
			   else
			   {
				   $gd_version_number = 0;
			   }
			}
			return $gd_version_number;
		}
}

?>